==> datafield-1.0.22.tar.gz: datafield/src/Plugin/DataField/FieldType/EntityReferenceItem.php <==
<?php

namespace Drupal\datafield\Plugin\DataField\FieldType;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\views\Views;

/**
 * Defines the 'entity_reference' entity field type.
 *
 * Supported settings (below the definition's 'settings' key) are:
 * - target_type: The entity type to reference. Required.
 *
 * @DataFieldType(
 *   id = "entity_reference",
 *   label = @Translation("Entity reference"),
 *   description = @Translation("An entity field containing an entity reference."),
 *   category = @Translation("Reference"),
 *   default_widget = "entity_reference_autocomplete",
 *   default_formatter = "entity_reference_label",
 *   listClass = "\Drupal\Core\Field\EntityReferenceFieldItemList",
 * )
 */
class EntityReferenceItem extends IntegerItem {
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getLabel() {
    return $this->t('Entity Reference');
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $field_settings, array $settings) {
    $field_settings += [
      'entity_reference_type' => 'node',
      'target_bundles' => 'page',
    ];
    if ($field_settings["entity_reference_type"] == 'user') {
      $field_settings["target_bundles"] = 'user';
    }
    $repository = \Drupal::service('entity_type.repository')
      ->getEntityTypeLabels(TRUE);
    $contentType = $repository['Content'] ?? [];
    $contentType['image'] = $this->t('Image');
    $entity_reference_type = $field_settings['entity_reference_type'];
    $field_form = [
      'label' => [
        '#type' => 'textfield',
        '#title' => $this->t('Label'),
        '#default_value' => $field_settings['label'] ?? ucfirst($settings["name"]),
      ],
      'required' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Required'),
        '#default_value' => $field_settings['required'] ?? FALSE,
      ],
      'entity_reference_type' => [
        '#type' => 'select',
        '#title' => $this->t('Entity reference type'),
        '#options' => $contentType,
        '#empty_option' => $this->t('- Select a field type -'),
        '#default_value' => $entity_reference_type,
        '#description' => $this->t('Save and back again to select bundle'),
      ],
    ];
    if (!empty($entity_reference_type)) {
      $entityTypeManager = \Drupal::entityTypeManager();
      $bundleInfo = \Drupal::service('entity_type.bundle.info');
      $view_storage = $entityTypeManager->getStorage('view');
      $displaysViewsRef = Views::getApplicableViews('entity_reference_display');
      $explode = explode(':', $entity_reference_type);
      $entity_type_id = end($explode);
      if ($entity_type_id == 'image') {
        $entity_type_id = 'file';
      }
      $entity_type = $entityTypeManager->getDefinition($entity_type_id);
      $titleBundle = $entity_type->getBundleLabel() ?? $this->t('Wait after select type');
      $bundles = $bundleInfo->getBundleInfo($entity_type_id);
      if (!empty($bundles)) {
        $bundle_options = [];
        foreach ($bundles as $bundle_name => $bundle_info) {
          $bundle_options[$bundle_name] = $bundle_info['label'];
        }
      }
      if (!empty($displaysViewsRef)) {
        // Filter views that list the entity type we want, and group
        // the separate displays by view.
        $optionsViewRef = [];
        foreach ($displaysViewsRef as $data) {
          [$view_id, $display_id] = $data;
          $view = $view_storage->load($view_id);
          if (in_array($view->get('base_table'), [
            $entity_type->getBaseTable(),
            $entity_type->getDataTable(),
          ])) {
            $display = $view->get('display');
            $optionsViewRef[$view_id . ':' . $display_id] = $view_id . ' - ' . $display[$display_id]['display_title'];
          }
        }
        if (!empty($optionsViewRef)) {
          $bundle_options['views'] = $optionsViewRef;
        }
      }
      $field_form['target_bundles'] = [
        '#type' => 'select',
        '#title' => $titleBundle,
        '#options' => $bundle_options ?? [],
        '#empty_option' => $this->t('- Select a bundle -'),
        '#default_value' => $field_settings['target_bundles'] ?? '',
      ];
      $tmp = explode(':', $field_settings['target_bundles']);
      // Check if the entity type is a view reference.
      if (count($tmp) > 1) {
        $field_form['view_arguments'] = [
          '#type' => 'textfield',
          '#title' => $this->t('View arguments'),
          '#default_value' => $field_settings['view_arguments'] ?? '',
          '#description' => $this->t('Provide a comma separated list of arguments to pass to the view.'),
        ];
      }
    }
    return $field_form;
  }

  /**
   * Get constraints.
   *
   * {@inheritdoc}
   */
  public function getConstraints(array $settings) {
    $constraints = [];
    if ($settings['required']) {
      $constraints['NotBlank'] = [];
    }
    return $constraints;
  }

  /**
   * Process widget form.
   *
   * {@inheritdoc}
   */
  public function setWidgetForm(&$widget, $field_settings, $entity) {
    $explode = explode(':', $field_settings["entity_reference_type"] ?? '');
    $reference_type = end($explode);
    if (empty($reference_type)) {
      $reference_type = 'node';
    }
    $widget['#target_type'] = $reference_type;
    $widget['#entity_type'] = $field_settings["entity_reference_type"] ?? 'node';
    $widget['#empty_option'] = $this->t('- Select -');
    $widget['#selection_handler'] = 'default';
    $bundle = $field_settings["target_bundles"] ?? '';
    $widget['#placeholder'] = implode(' ', [
      $this->t('Typing'),
      $field_settings["label"] ?? '',
    ]);
    $checkView = explode(':', $bundle);
    if (!empty($bundle)) {
      $widget['#bundle'] = $bundle;
      $widget['#autocreate']['bundle'] = $bundle;
    }
    $optionsView = [
      'target_type' => $reference_type,
      'entity' => $entity,
    ];
    $widget['#selection_settings'] = ['target_bundles' => [$bundle]];
    if ($field_settings["entity_reference_type"] == 'user') {
      $widget['#selection_settings'] = ['include_anonymous' => FALSE];
    }
    if ($reference_type != 'user') {
      $optionsView['target_bundles'] = [$bundle];
    }
    if (isset($widget['#default_value']) && empty($widget['#default_value'])) {
      unset($widget['#default_value']);
    }
    if (count($checkView) > 1) {
      [$view_name, $display_name] = $checkView;
      $view_arguments = $field_settings["view_arguments"] ?? '';
      $view = [
        'view_name' => $view_name,
        'display_name' => $display_name,
        'arguments' => explode(',', str_replace(' ', '', $view_arguments)),
      ];
      $widget['#selection_handler'] = 'views';
      $widget['#selection_settings']['view'] = $view;
      // Check if is views reference.
      $optionsView += [
        'handler' => 'views',
        'view' => $view,
      ];
    }
    $widget['#selection_settings']['options_view'] = $optionsView;
  }

}

==> datafield-2.0.1.tar.gz: datafield/src/Plugin/DataField/FieldType/EntityReferenceItem.php <==
<?php

namespace Drupal\datafield\Plugin\DataField\FieldType;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\datafield\TypedData\EntityDataDefinition;
use Drupal\views\Views;

/**
 * Defines the 'entity_reference' entity field type.
 *
 * Supported settings (below the definition's 'settings' key) are:
 * - target_type: The entity type to reference. Required.
 *
 * @DataFieldType(
 *   id = "entity_reference",
 *   label = @Translation("Entity reference"),
 *   description = @Translation("An entity field containing an entity reference."),
 *   category = @Translation("Reference"),
 *   default_widget = "entity_reference_autocomplete",
 *   default_formatter = "entity_reference_label",
 *   listClass = "\Drupal\Core\Field\EntityReferenceFieldItemList",
 * )
 */
class EntityReferenceItem extends IntegerItem {
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getLabel() {
    return $this->t('Entity Reference');
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $field_settings, array $settings) {
    $field_settings += [
      'entity_reference_type' => 'node',
      'target_bundles' => 'page',
    ];
    if ($field_settings["entity_reference_type"] == 'user') {
      $field_settings["target_bundles"] = 'user';
    }
    $repository = \Drupal::service('entity_type.repository')
      ->getEntityTypeLabels(TRUE);
    $contentType = !empty($repository) ? current($repository) : [];
    $contentType['image'] = $this->t('Image');
    $entity_reference_type = $field_settings['entity_reference_type'];
    $field_form = [
      'label' => [
        '#type' => 'textfield',
        '#title' => $this->t('Label'),
        '#default_value' => $field_settings['label'] ?? ucfirst($settings["name"]),
      ],
      'required' => [
        '#type' => 'checkbox',
        '#title' => $this->t('Required'),
        '#default_value' => $field_settings['required'] ?? FALSE,
      ],
      'entity_reference_type' => [
        '#type' => 'select',
        '#title' => $this->t('Entity reference type'),
        '#options' => $contentType,
        '#empty_option' => $this->t('- Select a field type -'),
        '#default_value' => $entity_reference_type,
        '#description' => $this->t('Save and back again to select bundle'),
      ],
    ];
    if (!empty($entity_reference_type)) {
      $entityTypeManager = \Drupal::entityTypeManager();
      $bundleInfo = \Drupal::service('entity_type.bundle.info');
      $view_storage = $entityTypeManager->getStorage('view');
      $displaysViewsRef = Views::getApplicableViews('entity_reference_display');
      $explode = explode(':', $entity_reference_type);
      $entity_type_id = end($explode);
      if ($entity_type_id == 'image') {
        $entity_type_id = 'file';
      }
      $entity_type = $entityTypeManager->getDefinition($entity_type_id);
      $titleBundle = $entity_type->getBundleLabel() ?? $this->t('Wait after select type');
      $bundles = $bundleInfo->getBundleInfo($entity_type_id);
      if (!empty($bundles)) {
        $bundle_options = [];
        foreach ($bundles as $bundle_name => $bundle_info) {
          $bundle_options[$bundle_name] = $bundle_info['label'];
        }
      }
      if (!empty($displaysViewsRef)) {
        // Filter views that list the entity type we want, and group
        // the separate displays by view.
        $optionsViewRef = [];
        foreach ($displaysViewsRef as $data) {
          [$view_id, $display_id] = $data;
          $view = $view_storage->load($view_id);
          if (in_array($view->get('base_table'), [
            $entity_type->getBaseTable(),
            $entity_type->getDataTable(),
          ])) {
            $display = $view->get('display');
            $optionsViewRef[$view_id . ':' . $display_id] = $view_id . ' - ' . $display[$display_id]['display_title'];
          }
        }
        if (!empty($optionsViewRef)) {
          $bundle_options['views'] = $optionsViewRef;
        }
      }
      $field_form['target_bundles'] = [
        '#type' => 'select',
        '#title' => $titleBundle,
        '#options' => $bundle_options ?? [],
        '#empty_option' => $this->t('- Select a bundle -'),
        '#default_value' => $field_settings['target_bundles'] ?? '',
      ];
      $tmp = explode(':', $field_settings['target_bundles']);
      // Check if the entity type is a view reference.
      if (count($tmp) > 1) {
        $field_form['view_arguments'] = [
          '#type' => 'textfield',
          '#title' => $this->t('View arguments'),
          '#default_value' => $field_settings['view_arguments'] ?? '',
          '#description' => $this->t('Provide a comma separated list of arguments to pass to the view.'),
        ];
      }
    }
    return $field_form;
  }

  /**
   * Get constraints.
   *
   * {@inheritdoc}
   */
  public function getConstraints(array $settings) {
    $constraints = [];
    if ($settings['required']) {
      $constraints['NotBlank'] = [];
    }
    return $constraints;
  }

  /**
   * Process widget form.
   *
   * {@inheritdoc}
   */
  public function setWidgetForm(&$widget, $field_settings, $entity) {
    $explode = explode(':', $field_settings["entity_reference_type"] ?? '');
    $reference_type = end($explode);
    if (empty($reference_type)) {
      $reference_type = 'node';
    }
    $widget['#target_type'] = $reference_type;
    $widget['#entity_type'] = $field_settings["entity_reference_type"] ?? 'node';
    $widget['#empty_option'] = $this->t('- Select -');
    $widget['#selection_handler'] = 'default';
    $bundle = $field_settings["target_bundles"] ?? '';
    $widget['#placeholder'] = implode(' ', [
      $this->t('Typing'),
      $field_settings["label"] ?? '',
    ]);
    $checkView = explode(':', $bundle);
    if (!empty($bundle)) {
      $widget['#bundle'] = $bundle;
      $widget['#autocreate']['bundle'] = $bundle;
    }
    $optionsView = [
      'target_type' => $reference_type,
      'entity' => $entity,
    ];
    $widget['#selection_settings'] = ['target_bundles' => [$bundle]];
    if ($field_settings["entity_reference_type"] == 'user') {
      $widget['#selection_settings'] = ['include_anonymous' => FALSE];
    }
    if ($reference_type != 'user') {
      $optionsView['target_bundles'] = [$bundle];
    }
    if (isset($widget['#default_value']) && empty($widget['#default_value'])) {
      unset($widget['#default_value']);
    }
    if (count($checkView) > 1) {
      [$view_name, $display_name] = $checkView;
      $view_arguments = $field_settings["view_arguments"] ?? '';
      $view = [
        'view_name' => $view_name,
        'display_name' => $display_name,
        'arguments' => explode(',', str_replace(' ', '', $view_arguments)),
      ];
      $widget['#selection_handler'] = 'views';
      $widget['#selection_settings']['view'] = $view;
      // Check if is views reference.
      $optionsView += [
        'handler' => 'views',
        'view' => $view,
      ];
    }
    $widget['#selection_settings']['options_view'] = $optionsView;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(array $settings) {
    $name = $settings['name'];
    $data_type = 'entity_datafield';
    return EntityDataDefinition::create($data_type)
      ->setLabel(new TranslatableMarkup('%name value', ['%name' => $name]))
      ->setRequired(FALSE);
  }

}
